#ifndef lint
static const char rcsid[] = "$Id: errorlog.c,v 1.1.1.1 1996/06/30 15:50:54 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Issaquah, WA
 *  29 January, 1996
 ****************************************************************************
 * $Log: errorlog.c,v $
 * Revision 1.1.1.1  1996/06/30 15:50:54  mtp
 * DCE threads for Linux V1.0
 *
 ****************************************************************************
 */
#include <syslog.h>
#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "errorlog.h"

/* 
 * Print a message and return to caller. Caller specifies "errnoflag" and
 * "priority".
 */
static void
log_doit(int errnoflag, int priority, const char *fmt, va_list ap)
{
	int		errno_save;
	char	buf[ERRLOG_C_STRLEN + 1];

        pthread_lock_global_np();
	errno_save = errno;		/* value caller might want printed */
	vsprintf(buf, fmt, ap);
	if (errnoflag)
		sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
	strcat(buf, "\n");
	syslog(priority, buf);
        pthread_unlock_global_np();
}


/* 
 * Initialize syslog(), if running as daemon.
 */
void
log_open( const char *ident, int option, int facility )
{
   pthread_lock_global_np();
   openlog(ident, option, facility);
   pthread_unlock_global_np();
}

/* 
 * Nonfatal error related to a system call.
 * Print a message with the system's errno value and return.
 **/
void
log_ret(const char *fmt, ...)
{
	va_list		ap;

	pthread_lock_global_np();
	va_start(ap, fmt);
	log_doit(1, LOG_ERR, fmt, ap);
	va_end(ap);
	pthread_unlock_global_np();
}

/* 
 * Fatal error related to a system call.  Print a message and terminate. 
 */
void
log_sys(const char *fmt, ...)
{
	va_list		ap;

	pthread_lock_global_np();
	va_start(ap, fmt);
	log_doit(1, LOG_ERR, fmt, ap);
	va_end(ap);
	exit(2);
	pthread_unlock_global_np();  /* NEVER GETS HERE */
}

/* 
 * Nonfatal error unrelated to a system call. Print a message and return.
 */
void
log_msg(const char *fmt, ...)
{
	va_list		ap;

	pthread_lock_global_np();
	va_start(ap, fmt);
	log_doit(0, LOG_ERR, fmt, ap);
	va_end(ap);
	pthread_unlock_global_np();
}

/* Fatal error unrelated to a system call.
 * Print a message and terminate. */

void
log_quit(const char *fmt, ...)
{
	va_list		ap;

	pthread_lock_global_np();
	va_start(ap, fmt);
	log_doit(0, LOG_ERR, fmt, ap);
	va_end(ap);
	exit(2);
	pthread_unlock_global_np();  /* NEVER GETS HERE */
}
