#ifndef lint
static const char rcsid[] = "$Id: test7.c,v 1.1.1.1 1996/06/30 15:50:54 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Issaquah, WA
 *  29 January, 1996
 ****************************************************************************
 * $Log: test7.c,v $
 * Revision 1.1.1.1  1996/06/30 15:50:54  mtp
 * DCE threads for Linux V1.0
 *
 ****************************************************************************
 */
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <utils.h>

#define ITERATIONS (1000000)

static void 
proc( struct timespec *tv )
{
   long sum = 0;
   int i, st;

   for(i = 0; i < ITERATIONS; i++ )
       sum += 1;
   
   st = pthread_delay_np( tv );
   CHECK(st, "pthread_delay_np()");

   for(i = 0; i < ITERATIONS; i++ )
       sum += 1;

   pthread_exit( (void *) SUCCESS );
}

static pthread_t  *th;

int 
main( int argc, char *argv[] )
{
   int i, st, exit_status, num_threads = 1;
   struct timespec tv;

   if( argc == 2 )
       num_threads = atoi( argv[1] );

   th = malloc_r( num_threads * sizeof( pthread_t ));

   tv.tv_sec = 1;
   tv.tv_nsec = rand() % 500000 + 100000;

   for(i = 0; i < num_threads; i++ )
   {
       st = create_joinable( &th[i], (thread_proc_t) proc, &tv );
	   CHECK(st, "create_joinable()");
   }

   for(i = 0; i < num_threads; i++ )
   {
       st = pthread_join( th[i], (void **) &exit_status );
	   CHECK(st, "pthread_join()");

	   pthread_lock_global_np();
       if( exit_status != SUCCESS )
           fprintf(stderr, "Failed to join with thread[%d]!\n", i + 2 );
	   else
		   printf("Joined with thread[%d]!\n", i + 2 );
	   pthread_unlock_global_np();
   }


   free_r( th );
   print_system_counters();

   /*
    * Do it all over again.
    */
   th = malloc_r( num_threads * sizeof( pthread_t ));

   tv.tv_sec = 1;
   tv.tv_nsec = rand() % 500000 + 100000;

   for(i = 0; i < num_threads; i++ )
   {
       st = create_joinable( &th[i], (thread_proc_t) proc, &tv );
	   CHECK(st, "create_joinable()");
   }

   for(i = 0; i < num_threads; i++ )
   {
       st = pthread_join( th[i], (void **) &exit_status );
	   CHECK(st, "pthread_join()");

	   pthread_lock_global_np();
       if( exit_status != SUCCESS )
           fprintf(stderr, "Failed to join with thread[%d]!\n", i + 2 );
	   else
		   printf("Joined with thread[%d]!\n", i + 2 );
	   pthread_unlock_global_np();
   }

   free_r( th );

   print_system_counters();
   return( EXIT_SUCCESS );
}
